#ifndef DEBTAGS_DOCUMENT_H
#define DEBTAGS_DOCUMENT_H

/*
 * Tagged Collection document for a Document-View editing architecture
 *
 * Copyright (C) 2003--2006  Enrico Zini <enrico@debian.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <sigc++/sigc++.h>

#include <ept/forward.h>
#include <ept/configuration/apt.h>
#include <ept/cache/apt/packages.h>
#include <ept/cache/debtags/tagmap.h>
#include <ept/cache/debtags/vocabulary.h>
#include <ept/predicate/factory.h>

#include <tagcoll/patch.h>
#include <tagcoll/coll/fast.h>
#if 0
#include <ept/cache/entity/package.h>
#include <ept/cache/entity/tag.h>
#include <ept/cache/component/packagetags.h>
#include <ept/cache/component/tags.h>

#include <tagcoll/InputMerger.h>
#include <tagcoll/ItemGrouper.h>

//#include <debtags/Utils.h>
#endif

#include "Mutex.h"
#include "Thread.h"


#include <vector>

class DebtagsDocument // : public Debtags::DebtagsSimple<aptFront::cache::entity::Package>
{
public:
	typedef SigC::Signal0<void> type_signal_changed;
	typedef SigC::Signal0<void> type_signal_reselected;
	typedef SigC::Signal0<void> type_signal_filename_changed;

	typedef ept::configuration::apt::cache::Aggregator Aggregator;
	typedef ept::configuration::apt::cache::Index Index;
	typedef ept::configuration::apt::cache::TagMap TagMap;
	typedef ept::configuration::apt::cache::Vocabulary Vocabulary;

	typedef ept::configuration::apt::cache::Package Package;
	typedef ept::configuration::apt::cache::Facet Facet;
	typedef ept::configuration::apt::cache::Tag Tag;

	typedef ept::predicate::Predicate<Package> Predicate;

protected:
#if 0
	class SpecialsGen : public Thread, public Debtags::Status
	{
	protected:
		DebtagsDocument& doc;
		Tagcoll::ItemGrouper<package, Tag> coll;
		bool _interrupt;

		virtual const char* threadTag() { return "SpecialsGen"; }

		virtual void* main() throw ();

	public:
		SpecialsGen(DebtagsDocument& doc) : doc(doc), _interrupt(false) {}
		virtual ~SpecialsGen() throw () {}

		void interrupt(bool val = true) throw () { _interrupt = val; }
		bool advance(int steps = 1) throw ();
	
		friend class DebtagsDocument;
	};
#endif

	Aggregator _agg;
	
	bool _changed;

	Package _current;

	tagcoll::Patch<Package, Tag> _lastPatch;
	std::vector<tagcoll::PatchList<Package, Tag> > _undoBuffer;
	unsigned int _undoTail;

	type_signal_changed _signal_changed;
	type_signal_reselected _signal_reselected;
	type_signal_filename_changed _signal_filename_changed;

	//OpSet<T> handlesToItems(const OpSet<int>& handles) const throw ();
	
#if 0
	SpecialsGen specialsGen;
#endif

	bool computeIntensive;

	Predicate _filter;
	
public:
	DebtagsDocument() throw ();
	virtual ~DebtagsDocument();

	Index& packages() { return _agg.index(); }
	const Index& packages() const { return _agg.index(); }

	TagMap& debtags() { return _agg.tagMap(); }
	const TagMap& debtags() const { return _agg.tagMap(); }

	Vocabulary& vocabulary() { return _agg.vocabulary(); }
	const Vocabulary& vocabulary() const { return _agg.vocabulary(); }

	const tagcoll::PatchList<Package, Tag> getPatch() const
	{
		return debtags().changes();
	}

	bool getComputeIntensive() const throw () { return computeIntensive; }
	void setComputeIntensive(bool val) throw ()
	{
		if (!computeIntensive && val)
		{
			computeIntensive = val;
#if 0
			//regenerateSpecials();
#endif
		} else
			computeIntensive = val;
	}
	
	//HandleMaker<T>& handles() throw () { return _handleMaker; }
	//Debtags::Packages& collection() throw () { return _collection; }
	//const Debtags::Packages& collection() const throw () { return _collection; }

	bool changed() const { return _changed; }

	Package& current() { return _current; }
	const Package& current() const { return _current; }

	std::set<Tag> currentTags() const
	{
		if (_current.valid())
			return debtags().getTagsOfItem(_current);
		else
			return std::set<Tag>();
	}

	void setCurrent(const Package& pkg);

	void resetFilter();
	void setFilter(const Predicate& p);

	template<typename OUT>
	void outputFiltered(OUT out)
	{
		if (_filter == ept::predicate::True<Package>())
			std::copy(_agg.index().begin(), _agg.index().end(), out);
		else
		{
			typeof(filteredRange(_agg.index().range(), _filter)) r =
				filteredRange(_agg.index().range(), _filter);
			std::copy(r.begin(), r.end(), out);
		}
	}

	// Apply a change, appending the reverse change in the undo buffer
	void applyChange(const tagcoll::Patch<Package, Tag>& change);
	void applyChange(const tagcoll::PatchList<Package, Tag>& change);

	bool canUndo() const;
	bool canRedo() const;
	bool canReapply(const Package& pkg) const;

	void undo();
	void redo();
	void reapplyToCurrent() { reapply(current()); }
	void reapply() { reapply(current()); }
	void reapply(const Package& pkg);

	//void load() throw (FileException, ParserException);
	void save() throw (wibble::exception::File);
	void send() throw (wibble::exception::File);

	//void output(TagcollConsumer<T>& consumer) throw ();
	//void output(TagcollConsumer<int>& consumer) throw ();

	// Results of the package filter
	//Debtags::PackageSet currentPackages;
	tagcoll::coll::Fast<Package, Tag> subCollection;

	//Debtags::Specials<Package> specials;
	enum { YES, NO, GENERATING } hasSpecials;
	Mutex specialsMutex;

	// Cached not-yet-tagged tags
	Tag nyt;
	Tag nytx[26];

	/**
	 * Fork a subthread to regenerate the specials.
	 *
	 * If a previous subthread is stull running, it is interrupted and restarted.
	 */
#if 0
	void regenerateSpecials() throw ();

	/**
	 * Request the specials to be fully generated
	 *
	 * Must be called when specialsMutex is locked.  On exit, the specials are
	 * guaranteed to have been generated.
	 */
	void wantSpecials() throw ();
#endif

	type_signal_changed signal_changed() throw () { return _signal_changed; }
	type_signal_reselected signal_reselected() throw () { return _signal_reselected; }
	type_signal_filename_changed signal_filename_changed() throw () { return _signal_filename_changed; }

	virtual void do_changed();
	virtual void do_reselected();

	friend class SpecialsGen;
};

// vim:set ts=4 sw=4:
#endif
